@@ -4,10 +4,11 @@ from urllib.parse import urlparse
 
 import gitlab
 import structlog
+from gitlab.base import RESTObjectList
 from gitlab.v4.objects import Project as GitlabProject
 from gitlab.v4.objects.merge_requests import ProjectMergeRequest as GitlabMergeRequest
 
-from .source import PullRequest, Repository, SourceLister
+from .source import Base, PullRequest, Repository
 
 log = structlog.get_logger(source="gitlab")
 
@@ -38,13 +39,32 @@ class GitlabRepository(Repository):
             }
         )
 
-    def find_open_pull_request(self, branch: str) -> Union[Any, None]:
+    def find_pull_request(self, branch: str) -> Union[Any, None]:
         log.debug("Listing merge requests", repo=str(self))
-        mrs = self._project.mergerequests.list(state="opened", source_branch=branch)
+        mrs = self._project.mergerequests.list(state="all", source_branch=branch)
         if len(mrs) == 0:
             return None
 
-        return mrs[0]
+        if isinstance(mrs, list):
+            return mrs[0]
+
+        if isinstance(mrs, RESTObjectList):
+            return mrs.next()
+
+        raise RuntimeError(
+            f"GitLab API returned an unexpected object {mrs.__class__.__name__}"
+        )
+
+    def has_file(self, path: str) -> bool:
+        try:
+            self._project.files.get(file_path=path, ref=self.base_branch)
+        except gitlab.exceptions.GitlabGetError as e:
+            if e.response_code == 404:
+                return False
+            else:
+                raise e
+
+        return True
 
     def has_successful_pr_build(self, identifier: GitlabMergeRequest) -> bool:
         pipelines = identifier.pipelines.list()
@@ -67,6 +87,15 @@ class GitlabRepository(Repository):
         )
         return True
 
+    def is_pr_closed(self, mr: GitlabMergeRequest) -> bool:
+        return mr.state == "closed" or mr.state == "locked"
+
+    def is_pr_merged(self, mr: GitlabMergeRequest) -> bool:
+        return mr.state == "merged"
+
+    def is_pr_open(self, mr: GitlabMergeRequest) -> bool:
+        return mr.state == "opened"
+
     def merge_pull_request(self, identifier: GitlabMergeRequest) -> None:
         log.debug("Merging merge request", repo=str(self), id=identifier.get_id())
         identifier.merge()
@@ -88,12 +117,13 @@ class GitlabRepository(Repository):
         return self.url
 
 
-class Gitlab(SourceLister):
+class Gitlab(Base):
     def __init__(self, url: str, private_token: str):
         self.client = gitlab.Gitlab(url, private_token=private_token)
         self.url = urlparse(url).netloc
 
     def list_repositories(self) -> list[Repository]:
+        log.debug("start fetching repositories")
         projects = self.client.projects.list(
             all=True, archived=False, min_access_level=30
         )
@@ -101,4 +131,5 @@ class Gitlab(SourceLister):
         for p in projects:
             repositories.append(GitlabRepository(project=p, url=self.url))  # type: ignore
 
+        log.debug("finished fetching repositories")
         return repositories
