@@ -73,24 +73,27 @@ _This pull request has been created by [rcmt](https://rcmt.readthedocs.io/)._
 
 
 class Repository:
+    """
+    Repository provides all methods needed to interact with a single repository of a
+    Source.
+    """
+
     def __str__(self):
         return f"{self.source}/{self.project}/{self.name}"
 
     @property
     def base_branch(self) -> str:
         """
-        Name of the base branch of this repository.
-
-        :return: str
+        :return: Name of the base branch of this repository.
+        :rtype: str
         """
         raise NotImplementedError("class does not implement Repository.base_branch()")
 
     @property
     def clone_url(self) -> str:
         """
-        Url to clone this repository.
-
-        :return: str
+        :return: Url to clone this repository.
+        :rtype: str
         """
         raise NotImplementedError("class does not implement Repository.clone_url()")
 
@@ -100,42 +103,93 @@ class Repository:
 
         :param branch: Name of the branch.
         :param pr: The pull request.
-        :return: None
+        :rtype: None
         """
         raise NotImplementedError(
             "class does not implement Repository.create_pull_request()"
         )
 
-    def find_open_pull_request(self, branch: str) -> Union[Any, None]:
+    def find_pull_request(self, branch: str) -> Union[Any, None]:
         """
         Finds and returns the pull request opened by rcmt.
 
         :param branch: Name of the branch from which a pull request has been created.
         :return: Implementation of this method should return None if no pull request is open. Any other value will be
                  passed to merge_pull_request() so it can identify which PR to merge.
+        :rtype: Any, None
         """
         raise NotImplementedError(
             "class does not implement Repository.has_open_pull_request()"
         )
 
+    def has_file(self, path: str) -> bool:
+        """
+        Checks if a file exists in a repository.
+
+        rcmt calls this function when matching repositories. This is more efficient than
+        checking out the whole repository to check if a file exists.
+
+        :param path: Path to a file or directory in the repository.
+        :return: Indicates if the files exists.
+        :rtype: bool
+        """
+        raise NotImplementedError("class does not implement Repository.has_file()")
+
     def has_successful_pr_build(self, identifier: Any) -> bool:
         """
         Checks if a pull request has passed all checks. rcmt will call merge_pull_request if this function returns
         True.
 
-        :param identifier: Data to identify the pull request as returned by find_open_pull_request.
-        :return: bool
+        :param identifier: Data to identify the pull request as returned by find_pull_request.
+        :return: Indicates if the build is successful.
+        :rtype: bool
         """
         raise NotImplementedError(
             "class does not implement Repository.has_successful_build()"
         )
 
+    def is_pr_closed(self, identifier: Any) -> bool:
+        """
+        Checks if a pull request has been closed by the user.
+        A pull request is closed if a user manually closes it without merging it.
+        If a user has closed a pull request without merging it, rcmt will not create a
+        new one.
+
+        :param identifier: Data to identify the pull request as returned by find_pull_request.
+        :return: Indicates that a pull request has been closed by the user.
+        :rtype: bool
+        """
+        raise NotImplementedError("class does not implement Repository.is_pr_closed()")
+
+    def is_pr_merged(self, identifier: Any) -> bool:
+        """
+        Checks if a pull request has been merged.
+        rcmt uses this information to determine if it can create new pull requests.
+
+        :param identifier: Data to identify the pull request as returned by find_pull_request.
+        :return: Indicates that a pull request has been merged.
+        :rtype: bool
+        """
+        raise NotImplementedError("class does not implement Repository.is_pr_merged()")
+
+    def is_pr_open(self, identifier: Any) -> bool:
+        """
+        Checks if a pull request is open.
+        rcmt will attempt to merge the pull request if this method returns true and
+        there are no new changes from packages.
+
+        :param identifier: Data to identify the pull request as returned by find_pull_request.
+        :return: Indicates that a pull request is open.
+        :rtype: bool
+        """
+        raise NotImplementedError("class does not implement Repository.is_pr_open()")
+
     def merge_pull_request(self, identifier: Any) -> None:
         """
         Merges a pull request.
 
-        :param identifier: Data to identify the pull request as returned by find_open_pull_request.
-        :return: None
+        :param identifier: Data to identify the pull request as returned by find_pull_request.
+        :rtype: None
         """
         raise NotImplementedError(
             "class does not implement Repository.merge_pull_request()"
@@ -144,9 +198,8 @@ class Repository:
     @property
     def name(self) -> str:
         """
-        Name of the repository.
-
-        :return: str
+        :return: Name of the repository.
+        :rtype: str
         """
         raise NotImplementedError("class does not implement Repository.name()")
 
@@ -154,17 +207,17 @@ class Repository:
         """
         Returns the date and time at which the pull request was created.
 
-        :param pr: The pull request identifier as returned by find_open_pull_request().
-        :return: datetime.datetime
+        :param pr: The pull request identifier as returned by find_pull_request().
+        :return: Date and time at which the pull request was created.
+        :rtype: datetime.datetime
         """
         raise NotImplementedError("class does not implement Repository.pr_created_at()")
 
     @property
     def project(self) -> str:
         """
-        Name of the project the repository belongs to.
-
-        :return: str
+        :return: Name of the project the repository belongs to.
+        :rtype: str
         """
 
         raise NotImplementedError("class does not implement Repository.project()")
@@ -172,19 +225,23 @@ class Repository:
     @property
     def source(self) -> str:
         """
-        Name of the source that hosts the repository.
-
-        :return: str
+        :return: Name of the source that hosts the repository.
+        :rtype: str
         """
         raise NotImplementedError("class does not implement Repository.source()")
 
 
-class Source:
-    pass
+class Base:
+    """
+    Base defines the methods every Source needs to implement.
 
+    """
 
-class SourceLister:
     def list_repositories(self) -> list[Repository]:
+        """
+        :return: List of all known repositories.
+        :rtype: list[rcmt.source.Repository]
+        """
         raise NotImplementedError(
             "class does not implement SourceLister.list_repositories()"
         )
